@@ -59,7 +59,29 @@ func (m *Manager) Save() *dbus.Error {
 	return dbusutil.ToError(err)
 }
 
-func (m *Manager) AssociateTouch(outputName, touchUUID string) *dbus.Error {
+func (m *Manager) AssociateTouch(outputName, touchSerial string) *dbus.Error {
+	var UUID string
+	for _, v := range m.Touchscreens {
+		if v.Serial == touchSerial {
+			UUID = v.UUID
+			break
+		}
+	}
+
+	if UUID == "" {
+		return dbusutil.ToError(errors.New("touchscreen not exists"))
+	}
+
+	monitor := m.getConnectedMonitors().GetByName(outputName)
+	if monitor == nil {
+		return dbusutil.ToError(errors.New("monitor not exists"))
+	}
+
+	err := m.associateTouch(monitor, UUID, false)
+	return dbusutil.ToError(err)
+}
+
+func (m *Manager) AssociateTouchByUUID(outputName, touchUUID string) *dbus.Error {
 	var UUID string
 	for _, v := range m.Touchscreens {
 		if v.UUID == touchUUID {
@@ -97,6 +119,7 @@ func (m *Manager) GetBrightness() (map[string]float64, *dbus.Error) {
 func (m *Manager) ListOutputNames() ([]string, *dbus.Error) {
 	logger.Debug("dbus call ListOutputNames")
 	var names []string
+
 	monitors := m.getConnectedMonitors()
 	for _, monitor := range monitors {
 		names = append(names, monitor.Name)
@@ -208,18 +231,23 @@ func (m *Manager) CanRotate() (bool, *dbus.Error) {
 	return true, nil
 }
 
+func (m *Manager) canSetBrightness(name string) bool {
+	//如果是龙芯集显，且不是内置显示器，则不支持调节亮度
+	if os.Getenv("CAN_SET_BRIGHTNESS") == "N" {
+		if m.builtinMonitor == nil || m.builtinMonitor.Name != name {
+			return false
+		}
+	}
+
+	return true
+}
+
 func (m *Manager) CanSetBrightness(outputName string) (bool, *dbus.Error) {
 	if outputName == "" {
 		return false, dbusutil.ToError(errors.New("monitor Name is err"))
 	}
 
-	//如果是龙芯集显，且不是内置显示器，则不支持调节亮度
-	if os.Getenv("CAN_SET_BRIGHTNESS") == "N" {
-		if m.builtinMonitor == nil || m.builtinMonitor.Name != outputName {
-			return false, nil
-		}
-	}
-	return true, nil
+	return m.canSetBrightness(outputName), nil
 }
 
 func (m *Manager) getBuiltinMonitor() *Monitor {
@@ -284,3 +312,7 @@ func (m *Manager) GetRealDisplayMode() (uint8, *dbus.Error) {
 
 	return mode, nil
 }
+
+func (m *Manager) SupportSetColorTemperature() (bool, *dbus.Error) {
+	return !(_inVM || _useWayland || !m.drmSupportGamma), nil
+}
