@@ -38,18 +38,17 @@ import (
 	dbus "github.com/godbus/dbus"
 	accounts "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.accounts"
 	notifications "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.notifications"
-	x "github.com/linuxdeepin/go-x11-client"
-	"github.com/linuxdeepin/startdde/display"
-	"github.com/linuxdeepin/startdde/iowait"
-	"github.com/linuxdeepin/startdde/watchdog"
-	wl_display "github.com/linuxdeepin/startdde/wl_display"
-	"github.com/linuxdeepin/startdde/wm_kwin"
-	"github.com/linuxdeepin/startdde/xsettings"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/gettext"
 	"github.com/linuxdeepin/go-lib/gsettings"
 	"github.com/linuxdeepin/go-lib/log"
 	"github.com/linuxdeepin/go-lib/proxy"
+	x "github.com/linuxdeepin/go-x11-client"
+	"github.com/linuxdeepin/startdde/display"
+	"github.com/linuxdeepin/startdde/iowait"
+	"github.com/linuxdeepin/startdde/watchdog"
+	"github.com/linuxdeepin/startdde/wm_kwin"
+	"github.com/linuxdeepin/startdde/xsettings"
 )
 
 var logger = log.NewLogger("startdde")
@@ -253,7 +252,7 @@ func greeterDisplayMain() {
 		os.Exit(1)
 	}
 	// TODO
-	display.Init(xConn, false)
+	display.Init(xConn, false, false)
 	logger.Debug("greeter mode")
 	service, err := dbusutil.NewSessionService()
 	if err != nil {
@@ -306,7 +305,7 @@ func main() {
 		logger.Info("in wayland mode")
 		_useWayland = true
 	}
-	display.Init(xConn, _useWayland)
+	display.Init(xConn, _useWayland, _inVM)
 	// TODO
 	recommendedScaleFactor = display.GetRecommendedScaleFactor()
 
@@ -339,6 +338,12 @@ func main() {
 		logger.Warning("start display part1 failed:", err)
 	}
 
+	// NOTE: always start pulseaudio
+	err = startPulseAudio()
+	if err != nil {
+		logger.Warning("failed to start pulseaudio:", err)
+	}
+
 	launchCoreComponents(sessionManager)
 
 	// 启动 display 模块的后一部分
@@ -410,11 +415,7 @@ loop:
 
 func doSetLogLevel(level log.Priority) {
 	logger.SetLogLevel(level)
-	if !_useWayland {
-		display.SetLogLevel(level)
-	} else {
-		wl_display.SetLogLevel(level)
-	}
+	display.SetLogLevel(level)
 	watchdog.SetLogLevel(level)
 }
 
