@@ -30,32 +30,35 @@ import (
 	"strconv"
 	"strings"
 	"sync"
+	"syscall"
 	"time"
 
-	powermanager "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.powermanager"
-
-	dbus "github.com/godbus/dbus"
+	"github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-api/soundutils"
 	daemon "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.daemon"
+	powermanager "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.powermanager"
+	sysbt "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.bluetooth"
 	ofdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.dbus"
 	login1 "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.login1"
+	systemd1 "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.systemd1"
+	gio "github.com/linuxdeepin/go-gir/gio-2.0"
+	"github.com/linuxdeepin/go-lib/appinfo/desktopappinfo"
+	"github.com/linuxdeepin/go-lib/cgroup"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/keyfile"
+	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/go-lib/procfs"
+	"github.com/linuxdeepin/go-lib/xdg/basedir"
 	x "github.com/linuxdeepin/go-x11-client"
 	"github.com/linuxdeepin/go-x11-client/ext/dpms"
-	"pkg.deepin.io/dde/api/soundutils"
-	"pkg.deepin.io/dde/startdde/autostop"
-	"pkg.deepin.io/dde/startdde/memchecker"
-	"pkg.deepin.io/dde/startdde/swapsched"
-	"pkg.deepin.io/dde/startdde/watchdog"
-	"pkg.deepin.io/dde/startdde/wm_kwin"
-	"pkg.deepin.io/dde/startdde/xcursor"
-	"pkg.deepin.io/dde/startdde/xsettings"
-	gio "pkg.deepin.io/gir/gio-2.0"
-	"pkg.deepin.io/lib/appinfo/desktopappinfo"
-	"pkg.deepin.io/lib/cgroup"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/keyfile"
-	"pkg.deepin.io/lib/log"
-	"pkg.deepin.io/lib/procfs"
-	"pkg.deepin.io/lib/xdg/basedir"
+	"github.com/linuxdeepin/startdde/autostop"
+	"github.com/linuxdeepin/startdde/keyring"
+	"github.com/linuxdeepin/startdde/memchecker"
+	"github.com/linuxdeepin/startdde/swapsched"
+	"github.com/linuxdeepin/startdde/watchdog"
+	"github.com/linuxdeepin/startdde/wm_kwin"
+	"github.com/linuxdeepin/startdde/xcursor"
+	"github.com/linuxdeepin/startdde/xsettings"
 )
 
 const (
@@ -80,16 +83,19 @@ type SessionManager struct {
 	cookies               map[string]chan time.Time
 	Stage                 int32
 	allowSessionDaemonRun bool
-	loginSession          *login1.Session
-	dbusDaemon            *ofdbus.DBus         // session bus daemon
+	loginSession          login1.Session
+	dbusDaemon            ofdbus.DBus          // session bus daemon
 	sigLoop               *dbusutil.SignalLoop // session bus signal loop
 	inhibitManager        InhibitManager
-	powerManager          *powermanager.PowerManager
+	powerManager          powermanager.PowerManager
+	sysBt                 sysbt.Bluetooth
 
 	CurrentSessionPath  dbus.ObjectPath
-	objLogin            *login1.Manager
-	objLoginSessionSelf *login1.Session
-	daemon              *daemon.Daemon
+	objLogin            login1.Manager
+	objLoginSessionSelf login1.Session
+	daemon              daemon.Daemon
+	inCallRequestLock   bool
+	inCallRequestLockMu sync.Mutex
 
 	//nolint
 	signals *struct {
@@ -148,6 +154,19 @@ func stopBAMFDaemon() {
 	}
 }
 
+func stopRedshift() {
+	bus, err := dbus.SessionBus()
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+	systemdManger := systemd1.NewManager(bus)
+	_, err = systemdManger.StopUnit(0, "redshift.service", "replace")
+	if err != nil {
+		logger.Warning("failed to stop redshift.service:", err)
+	}
+}
+
 func (m *SessionManager) prepareLogout(force bool) {
 	if !force {
 		err := autostop.LaunchAutostopScripts(logger)
@@ -162,11 +181,14 @@ func (m *SessionManager) prepareLogout(force bool) {
 	// quit at-spi-dbus-bus.service
 	quitAtSpiService()
 	stopBAMFDaemon()
-	sendMsgToUserExperModule(UserLogoutMsg)
-	quitObexSevice()
+	stopRedshift()
+	quitObexService()
+	//注销系统断开所有蓝牙连接
+	m.sysBt.DisconnectAudioDevices(0)
+
 	if !force && soundutils.CanPlayEvent(soundutils.EventDesktopLogout) {
+		// playLogoutSound 内部会退出 pulseaudio
 		playLogoutSound()
-		// PulseAudio should have quit
 	} else {
 		quitPulseAudio()
 	}
@@ -199,6 +221,9 @@ func (m *SessionManager) ForceLogout() *dbus.Error {
 }
 
 func (m *SessionManager) clearCurrentTty() {
+	if m.loginSession == nil {
+		return
+	}
 	vTNr, err := m.loginSession.VTNr().Get(0)
 	if err != nil {
 		logger.Warning("clearCurrentTty:", err)
@@ -221,8 +246,12 @@ func (m *SessionManager) logout(force bool) {
 }
 
 func (m *SessionManager) CanShutdown() (bool, *dbus.Error) {
-	can, err := m.powerManager.CanShutdown(0)
-	return can, dbusutil.ToError(err)
+	str, err := m.objLogin.CanPowerOff(0) // 当前能否关机
+	if err != nil {
+		logger.Warning(err)
+		return false, dbusutil.ToError(err)
+	}
+	return str == "yes", nil
 }
 
 func (m *SessionManager) Shutdown() *dbus.Error {
@@ -233,7 +262,6 @@ func (m *SessionManager) Shutdown() *dbus.Error {
 func (m *SessionManager) prepareShutdown(force bool) {
 	killSogouImeWatchdog()
 	stopBAMFDaemon()
-	sendMsgToUserExperModule(UserShutdownMsg)
 	if !force {
 		preparePlayShutdownSound()
 	}
@@ -276,16 +304,22 @@ func (m *SessionManager) shutdown(force bool) {
 	if _gSettingsConfig.needQuickBlackScreen {
 		setDPMSMode(false)
 	}
-	err = m.objLoginSessionSelf.Terminate(0)
-	if err != nil {
-		logger.Warning("failed to terminate session self:", err)
+	if m.objLoginSessionSelf != nil {
+		err = m.objLoginSessionSelf.Terminate(0)
+		if err != nil {
+			logger.Warning("failed to terminate session self:", err)
+		}
 	}
 	os.Exit(0)
 }
 
 func (m *SessionManager) CanReboot() (bool, *dbus.Error) {
-	can, err := m.powerManager.CanReboot(0)
-	return can, dbusutil.ToError(err)
+	str, err := m.objLogin.CanReboot(0) // 当前能否重启
+	if err != nil {
+		logger.Warning(err)
+		return false, dbusutil.ToError(err)
+	}
+	return str == "yes", nil
 }
 
 func (m *SessionManager) Reboot() *dbus.Error {
@@ -320,16 +354,34 @@ func (m *SessionManager) reboot(force bool) {
 	if _gSettingsConfig.needQuickBlackScreen {
 		setDPMSMode(false)
 	}
-	err = m.objLoginSessionSelf.Terminate(0)
-	if err != nil {
-		logger.Warning("failed to terminate session self:", err)
+	if m.objLoginSessionSelf != nil {
+		err = m.objLoginSessionSelf.Terminate(0)
+		if err != nil {
+			logger.Warning("failed to terminate session self:", err)
+		}
 	}
 	os.Exit(0)
 }
 
 func (m *SessionManager) CanSuspend() (bool, *dbus.Error) {
-	can, err := m.powerManager.CanSuspend(0)
-	return can, dbusutil.ToError(err)
+	if os.Getenv("POWER_CAN_SLEEP") == "0" {
+		logger.Info("can not Suspend, env POWER_CAN_SLEEP == 0")
+		return false, nil
+	}
+	can, err := m.powerManager.CanSuspend(0) // 是否支持待机
+	if err != nil {
+		logger.Warning(err)
+		return false, dbusutil.ToError(err)
+	}
+	if can {
+		str, err := m.objLogin.CanSuspend(0) // 当前能否待机
+		if err != nil {
+			logger.Warning(err)
+			return false, dbusutil.ToError(err)
+		}
+		return str == "yes", nil
+	}
+	return false, nil
 }
 
 func (m *SessionManager) RequestSuspend() *dbus.Error {
@@ -341,20 +393,44 @@ func (m *SessionManager) RequestSuspend() *dbus.Error {
 		return nil
 	}
 
+	// 使用窗管接口进行黑屏处理
+	if _gSettingsConfig.needQuickBlackScreen {
+		logger.Info("request wm blackscreen effect")
+		cmd := exec.Command("/bin/bash", "-c", "dbus-send --print-reply --dest=org.kde.KWin /BlackScreen org.kde.kwin.BlackScreen.setActive boolean:true")
+		error := cmd.Run()
+		if error != nil {
+			logger.Warning("wm blackscreen failed")
+		}
+	}
+
+	logger.Info("login1 start suspend")
 	err = m.objLogin.Suspend(0, false)
 	if err != nil {
 		logger.Warning("failed to suspend:", err)
 	}
 
-	if _gSettingsConfig.needQuickBlackScreen {
-		setDPMSMode(false)
-	}
 	return nil
 }
 
 func (m *SessionManager) CanHibernate() (bool, *dbus.Error) {
-	can, err := m.powerManager.CanHibernate(0)
-	return can, dbusutil.ToError(err)
+	if os.Getenv("POWER_CAN_SLEEP") == "0" {
+		logger.Info("can not Hibernate, env POWER_CAN_SLEEP == 0")
+		return false, nil
+	}
+	can, err := m.powerManager.CanHibernate(0) // 是否支持休眠
+	if err != nil {
+		logger.Warning(err)
+		return false, dbusutil.ToError(err)
+	}
+	if can {
+		str, err := m.objLogin.CanHibernate(0) // 当前能否休眠
+		if err != nil {
+			logger.Warning(err)
+			return false, dbusutil.ToError(err)
+		}
+		return str == "yes", nil
+	}
+	return false, nil
 }
 
 func (m *SessionManager) RequestHibernate() *dbus.Error {
@@ -433,6 +509,7 @@ func (m *SessionManager) SetLocked(sender dbus.Sender, value bool) *dbus.Error {
 }
 
 func (m *SessionManager) setLocked(value bool) {
+	logger.Debug("call setLocked", value)
 	m.mu.Lock()
 	if m.Locked != value {
 		m.Locked = value
@@ -460,14 +537,6 @@ func (m *SessionManager) setLocked(value bool) {
 	} else {
 		logger.Warning("watchdogManager is nil")
 	}
-
-	if !value && m.loginSession != nil {
-		// unlock
-		err := m.loginSession.Unlock(0)
-		if err != nil {
-			logger.Warning("failed to unlock login session:", err)
-		}
-	}
 }
 
 func (m *SessionManager) getLocked() bool {
@@ -498,21 +567,32 @@ func (m *SessionManager) initSession() {
 
 	sysSigLoop := dbusutil.NewSignalLoop(sysBus, 10)
 	sysSigLoop.Start()
-	m.objLoginSessionSelf.InitSignalExt(sysSigLoop, true)
-	err = m.objLoginSessionSelf.Active().ConnectChanged(func(hasValue bool, active bool) {
-		logger.Debug("session status changed:", hasValue, active)
-		if hasValue && !active {
-			isPreparingForSleep, _ := m.objLogin.PreparingForSleep().Get(0)
-			if !isPreparingForSleep {
-				err = m.objLoginSessionSelf.Lock(0)
-				if err != nil {
-					logger.Warning("failed to Lock current session:", err)
+	if m.objLoginSessionSelf != nil {
+		m.objLoginSessionSelf.InitSignalExt(sysSigLoop, true)
+		err = m.objLoginSessionSelf.Active().ConnectChanged(func(hasValue bool, active bool) {
+			logger.Debugf("session status changed hasValue: %v, active: %v", hasValue, active)
+			if !hasValue {
+				return
+			}
+
+			if active {
+				// 变活跃
+			} else {
+				// 变不活跃
+				isPreparingForSleep, _ := m.objLogin.PreparingForSleep().Get(0)
+				// 待机时不在这里锁屏
+				if !isPreparingForSleep {
+					logger.Debug("call objLoginSessionSelf.Lock")
+					err = m.objLoginSessionSelf.Lock(0)
+					if err != nil {
+						logger.Warning("failed to Lock current session:", err)
+					}
 				}
 			}
+		})
+		if err != nil {
+			logger.Warning("failed to connect Active changed:", err)
 		}
-	})
-	if err != nil {
-		logger.Warning("failed to connect Active changed:", err)
 	}
 	if _gSettingsConfig.swapSchedEnabled {
 		m.initSwapSched()
@@ -534,6 +614,9 @@ func (m *SessionManager) initSwapSched() {
 		return
 	}
 
+	if m.objLoginSessionSelf == nil {
+		return
+	}
 	sessionID, err := m.objLoginSessionSelf.Id().Get(0)
 	if err != nil {
 		logger.Warning(err)
@@ -606,13 +689,17 @@ func newSessionManager(service *dbusutil.Service) *SessionManager {
 	dbusDaemon.InitSignalExt(sigLoop, true)
 	objLogin := login1.NewManager(sysBus)
 	powerManager := powermanager.NewPowerManager(sysBus)
+	sysBt := sysbt.NewBluetooth(sysBus)
+	var objLoginSessionSelf login1.Session
 	sessionPath, err := objLogin.GetSessionByPID(0, 0)
 	if err != nil {
-		panic(fmt.Errorf("get session path failed: %s", err))
-	}
-	objLoginSessionSelf, err := login1.NewSession(sysBus, sessionPath)
-	if err != nil {
-		panic(fmt.Errorf("new Login1 session failed: %s", err))
+		logger.Warning("failed to get current session path:", err)
+	} else {
+		logger.Info("session path:", sessionPath)
+		objLoginSessionSelf, err = login1.NewSession(sysBus, sessionPath)
+		if err != nil {
+			logger.Warning("login1.NewSession err:", err)
+		}
 	}
 
 	m := &SessionManager{
@@ -622,9 +709,16 @@ func newSessionManager(service *dbusutil.Service) *SessionManager {
 		objLogin:            objLogin,
 		objLoginSessionSelf: objLoginSessionSelf,
 		powerManager:        powerManager,
+		sysBt:               sysBt,
 		dbusDaemon:          dbusDaemon,
 		daemon:              daemon.NewDaemon(sysBus),
 	}
+
+	// 此处将init的操作提前，避免SessionManager对象被创建了，相关属性值还没有被初始化
+	// 如果其它进程读取了非法的属性值， dbus连接会被关闭，导致startdde启动失败
+	m.initSession()
+	m.init()
+
 	return m
 }
 
@@ -639,6 +733,8 @@ func (m *SessionManager) init() {
 	m.CurrentSessionPath, err = getCurSessionPath()
 	if err != nil {
 		logger.Warning("failed to get current session path:", err)
+		// 要保证 CurrentSessionPath 属性值的合法性
+		m.CurrentSessionPath = "/"
 	}
 
 	m.initInhibitManager()
@@ -949,52 +1045,6 @@ func initQtThemeConfig() error {
 	return nil
 }
 
-const (
-	UserExperServiceName = "com.deepin.userexperience.Daemon"
-	UserExperPath        = "/com/deepin/userexperience/Daemon"
-	UserLoginMsg         = "login"
-	UserLogoutMsg        = "logout"
-	UserShutdownMsg      = "shutdown"
-
-	UserExperOpenApp  = "openapp"
-	UserExperCloseApp = "closeapp"
-
-	UserExperCLoseAppChanInitLen = 24
-)
-
-type UeMessageItem struct {
-	Path, Name, Id string
-}
-
-func sendMsgToUserExperModule(msg string) {
-	if isCommunity() {
-		return
-	}
-	// send message to user experience module
-	// first send will active the services
-	bus, err := dbus.SystemBus()
-	ch := make(chan struct{})
-	if err == nil {
-		go func() {
-			userexp := bus.Object(UserExperServiceName, UserExperPath)
-			err = userexp.Call(UserExperServiceName+".SendLogonData", 0, msg).Err
-			if err != nil {
-				logger.Warningf("failed to call %s.SendLogonData, %v", UserExperServiceName, err)
-			} else {
-				logger.Infof("send %s message to ue module", msg)
-			}
-			close(ch)
-		}()
-		select {
-		case <-ch:
-		case <-time.After(1 * time.Second):
-			logger.Debug("sendMsgToUserExperModule timed out!")
-		}
-	} else {
-		logger.Warning(err)
-	}
-}
-
 func (m *SessionManager) start(xConn *x.Conn, sysSignalLoop *dbusutil.SignalLoop, service *dbusutil.Service) *SessionManager {
 	defer func() {
 		if err := recover(); err != nil {
@@ -1002,8 +1052,7 @@ func (m *SessionManager) start(xConn *x.Conn, sysSignalLoop *dbusutil.SignalLoop
 			return
 		}
 	}()
-	m.initSession()
-	m.init()
+
 	if _options.noXSessionScripts {
 		runScript01DeepinProfileFaster()
 		runScript30X11CommonXResourcesFaster()
@@ -1029,10 +1078,15 @@ func (m *SessionManager) start(xConn *x.Conn, sysSignalLoop *dbusutil.SignalLoop
 	go startObexService()
 	m.launchDDE()
 
-	go setLeftPtrCursor()
+	go func() {
+		setLeftPtrCursor()
+		err := keyring.CheckLogin()
+		if err != nil {
+			logger.Warning("Failed to init keyring:", err)
+		}
+	}()
 	time.AfterFunc(3*time.Second, _startManager.listenAutostartFileEvents)
 	go m.launchAutostart()
-	sendMsgToUserExperModule(UserLoginMsg)
 
 	if m.loginSession != nil {
 		m.loginSession.InitSignalExt(sysSignalLoop, true)
@@ -1139,7 +1193,7 @@ func setLeftPtrCursor() {
 	}
 }
 
-func getLoginSession() (*login1.Session, error) {
+func getLoginSession() (login1.Session, error) {
 	sysBus, err := dbus.SystemBus()
 	if err != nil {
 		return nil, err
@@ -1159,20 +1213,111 @@ func getLoginSession() (*login1.Session, error) {
 
 func (m *SessionManager) handleLoginSessionLock() {
 	logger.Debug("login session lock")
-	err := m.RequestLock()
-	if err != nil {
-		logger.Warning("failed to request lock:", err)
-	}
+	go func() {
+		// 在特殊情况下，比如用 dde-switchtogreeter 命令切换到 greeter, 即切换到其他 tty,
+		// RequestLock 方法不能立即返回。
+
+		// 如果已经锁定，则立即返回
+		locked := m.getLocked()
+		if locked {
+			logger.Debug("handleLoginSessionLock locked is true, return")
+			return
+		}
+
+		// 使用 m.inCallRequestLock 防止同时多次调用 RequestLock，以简化状况。
+		m.inCallRequestLockMu.Lock()
+		if m.inCallRequestLock {
+			m.inCallRequestLockMu.Unlock()
+			// 如果已经发出一个锁屏请求，则立即返回。
+			logger.Debug("handleLoginSessionLock inCall is true, return")
+			return
+		}
+		m.inCallRequestLock = true
+		m.inCallRequestLockMu.Unlock()
+
+		t0 := time.Now()
+		logger.Debug("handleLoginSessionLock call RequestLock begin")
+		err := m.RequestLock()
+		elapsed := time.Since(t0)
+		logger.Debugf("handleLoginSessionLock call RequestLock end, cost: %v", elapsed)
+		if err != nil {
+			logger.Warning("failed to request lock:", err)
+		}
+
+		m.inCallRequestLockMu.Lock()
+		m.inCallRequestLock = false
+		m.inCallRequestLockMu.Unlock()
+	}()
+
 }
 
 func (m *SessionManager) handleLoginSessionUnlock() {
+	if m.loginSession == nil {
+		return
+	}
 	logger.Debug("login session unlock")
+	isActive, err := m.loginSession.Active().Get(0)
+	if err != nil {
+		logger.Warning("failed to get session property Active:", err)
+		return
+	}
+
+	if isActive {
+		// 图形界面 session 活跃
+		m.emitSignalUnlock()
+	} else {
+		// 图形界面 session 不活跃，此时采用 kill 掉锁屏前端的特殊操作，
+		// 如果不 kill 掉它，则一旦 session 变活跃，锁屏前端会在很近的时间内执行锁屏和解锁，
+		// 会使用系统通知报告锁屏失败。 而在此种特殊情况下 kill 掉它，并不会造成明显问题。
+		err = m.killLockFront()
+		if err != nil {
+			logger.Warning("failed to kill lock front:", err)
+			m.emitSignalUnlock()
+		} else {
+			m.setLocked(false)
+		}
+	}
+}
+
+// 发送 Unlock 信号，前端锁屏程序收到后会立即解锁。
+func (m *SessionManager) emitSignalUnlock() {
+	logger.Debug("emit signal Unlock")
 	err := m.service.Emit(m, "Unlock")
 	if err != nil {
 		logger.Warning("failed to emit signal Unlock:", err)
 	}
 }
 
+// kill 掉前端锁屏程序
+func (m *SessionManager) killLockFront() error {
+	sessionBus := m.service.Conn()
+	dbusDaemon := ofdbus.NewDBus(sessionBus)
+	owner, err := dbusDaemon.GetNameOwner(0, "com.deepin.dde.lockFront")
+	if err != nil {
+		return err
+	}
+
+	pid, err := m.service.GetConnPID(owner)
+	if err != nil {
+		return err
+	}
+	logger.Debugf("kill lock front owner: %v, pid: %v", owner, pid)
+
+	p, err := os.FindProcess(int(pid))
+	// linux 系统下这里不会失败
+	if err != nil {
+		return err
+	}
+
+	// 发送 SIGTERM 礼貌的退出信号
+	err = p.Signal(syscall.SIGTERM)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 func setDPMSMode(on bool) {
 	var err error
 	if _useWayland {
@@ -1195,9 +1340,11 @@ func setDPMSMode(on bool) {
 }
 
 func (m *SessionManager) doLogout(force bool) {
-	err := m.objLoginSessionSelf.Terminate(0)
-	if err != nil {
-		logger.Warning("LoginSessionSelf Terminate failed:", err)
+	if m.objLoginSessionSelf != nil {
+		err := m.objLoginSessionSelf.Terminate(0)
+		if err != nil {
+			logger.Warning("LoginSessionSelf Terminate failed:", err)
+		}
 	}
 	os.Exit(0)
 }
@@ -1224,7 +1371,6 @@ func startAtSpiService() {
 }
 
 func startObexService() {
-	time.Sleep(3 * time.Second)
 	logger.Debugf("starting %s...", obexService)
 	err := exec.Command("systemctl", "--user", "--runtime", "unmask", obexService).Run()
 	if err != nil {
@@ -1256,7 +1402,7 @@ func quitAtSpiService() {
 	}
 }
 
-func quitObexSevice() {
+func quitObexService() {
 	logger.Debugf("quitting %s...", obexService)
 	// mask obex.service
 	out, err := exec.Command("systemctl", "--user", "--runtime", "--now", "mask",
