@@ -21,7 +21,6 @@ package main
 
 import (
 	"bufio"
-	"context"
 	"errors"
 	"fmt"
 	"io"
@@ -38,16 +37,17 @@ import (
 	dbus "github.com/godbus/dbus"
 	daemonApps "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.apps"
 	systemPower "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.power"
+	proxy "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.proxy"
 	x "github.com/linuxdeepin/go-x11-client"
-	"pkg.deepin.io/dde/startdde/swapsched"
-	"pkg.deepin.io/gir/gio-2.0"
-	"pkg.deepin.io/lib/appinfo"
-	"pkg.deepin.io/lib/appinfo/desktopappinfo"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/gsettings"
-	"pkg.deepin.io/lib/keyfile"
-	"pkg.deepin.io/lib/strv"
-	"pkg.deepin.io/lib/xdg/basedir"
+	"github.com/linuxdeepin/startdde/swapsched"
+	"github.com/linuxdeepin/go-gir/gio-2.0"
+	"github.com/linuxdeepin/go-lib/appinfo"
+	"github.com/linuxdeepin/go-lib/appinfo/desktopappinfo"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/gsettings"
+	"github.com/linuxdeepin/go-lib/keyfile"
+	"github.com/linuxdeepin/go-lib/strv"
+	"github.com/linuxdeepin/go-lib/xdg/basedir"
 )
 
 //go:generate dbusutil-gen em -type StartManager,SessionManager,Inhibitor
@@ -82,7 +82,7 @@ type StartManager struct {
 	service             *dbusutil.Service
 	userAutostartPath   string
 	delayHandler        *mapDelayHandler
-	daemonApps          *daemonApps.Apps
+	daemonApps          daemonApps.Apps
 	restartTimeMap      map[string]time.Time
 	restartTimeMapMu    sync.Mutex
 	proxyChainsConfFile string
@@ -92,11 +92,11 @@ type StartManager struct {
 	appsUseProxy        strv.Strv
 	appsDisableScaling  strv.Strv
 	mu                  sync.Mutex
-	appClose            chan *UeMessageItem
+	appProxy            proxy.App
 	launchedHooks       []string
 
 	NeededMemory     uint64
-	systemPower      *systemPower.Power
+	systemPower      systemPower.Power
 	cpuFreqAdjustMap map[string]int32
 
 	//nolint
@@ -189,8 +189,6 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 
 	m.appsDir = getAppDirs()
 	m.settings = gio.NewSettings(gSchemaLauncher)
-	m.appClose = make(chan *UeMessageItem, UserExperCLoseAppChanInitLen)
-
 	m.appsUseProxy = m.settings.GetStrv(gKeyAppsUseProxy)
 	m.appsDisableScaling = m.settings.GetStrv(gKeyAppsDisableScaling)
 
@@ -210,11 +208,6 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 		logger.Debug("update ", key)
 	})
 
-	err := m.listenAppCloseEvent()
-	if err != nil {
-		logger.Warning(err)
-	}
-
 	m.proxyChainsConfFile = filepath.Join(basedir.GetUserConfigDir(), "deepin", "proxychains.conf")
 	m.proxyChainsBin, _ = exec.LookPath(proxychainsBinary)
 	logger.Debugf("startManager proxychain confFile %q, bin: %q", m.proxyChainsConfFile, m.proxyChainsBin)
@@ -230,6 +223,7 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 
 	m.daemonApps = daemonApps.NewApps(sysBus)
 	m.systemPower = systemPower.NewPower(sysBus)
+	m.appProxy = proxy.NewApp(sysBus)
 	m.cpuFreqAdjustMap = m.getCpuFreqAdjustMap(cpuFreqAdjustFile)
 	return m
 }
@@ -330,7 +324,7 @@ func (m *StartManager) launchAppWithOptions(desktopFile string, timestamp uint32
 
 	// mark app launched
 	if m.daemonApps != nil {
-		err := m.daemonApps.MarkLaunched(0, desktopFile)
+		err := m.daemonApps.LaunchedRecorder().MarkLaunched(0, desktopFile)
 		if err != nil {
 			logger.Warning(err)
 		}
@@ -369,7 +363,7 @@ func (m *StartManager) launchAppAction(desktopFile, action string, timestamp uin
 	}
 	// mark app launched
 	if m.daemonApps != nil {
-		err := m.daemonApps.MarkLaunched(0, desktopFile)
+		err := m.daemonApps.LaunchedRecorder().MarkLaunched(0, desktopFile)
 		if err != nil {
 			logger.Warning(err)
 		}
@@ -475,10 +469,6 @@ func (m *StartManager) getAppIdByFilePath(file string) string {
 }
 
 func (m *StartManager) shouldUseProxy(id string) bool {
-	// check if need ignore use proxy
-	if ignoreUseProxy(id) {
-		return false
-	}
 	m.mu.Lock()
 	if !m.appsUseProxy.Contains(id) {
 		m.mu.Unlock()
@@ -486,17 +476,16 @@ func (m *StartManager) shouldUseProxy(id string) bool {
 	}
 	m.mu.Unlock()
 
-	if _, err := os.Stat(m.proxyChainsConfFile); err != nil {
+	msg, err := m.appProxy.GetProxy(0)
+	if err != nil {
+		logger.Warningf("cant get proxy, err: %v", err)
+		return false
+	}
+	if msg == "" {
+		logger.Debug("dont have proxy settings, will not use proxy")
 		return false
 	}
 
-	if m.proxyChainsBin == "" {
-		// try get proxyChainsBin again
-		m.proxyChainsBin, _ = exec.LookPath(proxychainsBinary)
-		if m.proxyChainsBin == "" {
-			return false
-		}
-	}
 	return true
 }
 
@@ -513,7 +502,6 @@ type IStartCommand interface {
 
 func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp uint32,
 	files []string, iStartCmd IStartCommand, cmdName string) error {
-
 	// maximum RAM unit is MB
 	maxRAM, _ := appInfo.GetUint64(desktopappinfo.MainSection, "X-Deepin-MaximumRAM")
 	// unit is MB/s
@@ -524,7 +512,6 @@ func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp
 	logger.Debug("launch: desktopFile is", desktopFile)
 	var err error
 	var cmdPrefixes []string
-	var cmdSuffixes []string
 	var uiApp *swapsched.UIApp
 
 	err = m.enableCpuFreqLock(desktopFile)
@@ -555,19 +542,6 @@ func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp
 
 	appId := m.getAppIdByFilePath(desktopFile)
 	if appId != "" {
-		if m.shouldUseProxy(appId) {
-			logger.Debug("launch: use proxy")
-			if supportProxyServerOption(appId) {
-				proxyServerUrl, err := getProxyServerUrl()
-				if err == nil {
-					cmdSuffixes = append(cmdSuffixes, "--proxy-server="+proxyServerUrl)
-				} else {
-					logger.Warning("failed to get google chrome proxy server url:", err)
-				}
-			} else {
-				cmdPrefixes = append(cmdPrefixes, m.proxyChainsBin, "-f", m.proxyChainsConfFile)
-			}
-		}
 		if m.shouldDisableScaling(appId) {
 			logger.Debug("launch: disable scaling")
 			gs := gio.NewSettings("com.deepin.xsettings")
@@ -589,15 +563,18 @@ func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp
 		logger.Debug("cmd prefixes:", cmdPrefixes)
 		ctx.SetCmdPrefixes(cmdPrefixes)
 	}
-	if len(cmdSuffixes) > 0 {
-		logger.Debug("cmd suffixes:", cmdSuffixes)
-		ctx.SetCmdSuffixes(cmdSuffixes)
-	}
 
 	if appInfo.IsDesktopOverrideExecSet() {
 		logger.Debug("cmd override exec:", appInfo.GetDesktopOverrideExec())
 	}
 
+	logger.Infof("app id %v check use app proxy", appInfo.GetId())
+	if m.shouldUseProxy(appInfo.GetId()) {
+		env := removeProxy(os.Environ())
+		logger.Infof("app %v use app proxy, clear proxy env, env: %v", appInfo.GetId(), env)
+		ctx.SetEnv(env)
+	}
+
 	cmd, err := iStartCmd.StartCommand(files, ctx)
 
 	// exec launched hooks
@@ -607,47 +584,9 @@ func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp
 	}
 	go m.execLaunchedHooks(desktopFile, cGroupName)
 
-	item := &UeMessageItem{Path: appInfo.GetFileName(), Name: appInfo.GetName(), Id: appInfo.GetId()}
-	go sendAppDataMsgToUserExperModule(UserExperOpenApp, item)
-
 	return m.waitCmd(appInfo, cmd, err, uiApp, cmdName)
 }
 
-func (m *StartManager) listenAppCloseEvent() error {
-	go func() {
-		for item := range m.appClose {
-			if item != nil {
-				item := &UeMessageItem{Path: item.Path, Name: item.Name, Id: item.Id}
-				go sendAppDataMsgToUserExperModule(UserExperCloseApp, item)
-			}
-		}
-	}()
-
-	return nil
-}
-
-func sendAppDataMsgToUserExperModule(msg string, item *UeMessageItem) {
-	// 社区版不收集数据
-	if isCommunity() {
-		return
-	}
-	// send open app and close app message to user experience module
-	bus, err := dbus.SystemBus()
-	if err == nil {
-		userexp := bus.Object(UserExperServiceName, UserExperPath)
-		ctx, cancelFn := context.WithTimeout(context.Background(), 10*time.Second)
-		defer cancelFn()
-		err = userexp.CallWithContext(ctx, UserExperServiceName+".SendAppStateData", 0, msg, item.Path, item.Name, item.Id).Err
-		if err != nil {
-			logger.Warningf("failed to call %s.SendAppStateData, %v", UserExperServiceName, err)
-		} else {
-			logger.Infof("send %s message, app info %s/%s/%s", msg, item.Path, item.Name, item.Id)
-		}
-	} else {
-		logger.Warning(err)
-	}
-}
-
 func newDesktopAppInfoFromFile(filename string) (*desktopappinfo.DesktopAppInfo, error) {
 	dai, err := desktopappinfo.NewDesktopAppInfoFromFile(filename)
 	if err != nil {
@@ -718,21 +657,26 @@ func (m *StartManager) waitCmd(appInfo *desktopappinfo.DesktopAppInfo, cmd *exec
 	if uiApp != nil {
 		swapSchedDispatcher.AddApp(uiApp)
 	}
-
 	if err != nil {
 		return err
 	}
 
 	go func() {
-		err := cmd.Wait()
-
-		// send app close info to ue module
-		// we did not care the program exit normal or not
+		// check if should use new proxy
+		// check if app info is empty
 		if appInfo != nil {
-			item := &UeMessageItem{Path: appInfo.GetFileName(), Name: appInfo.GetName(), Id: appInfo.GetId()}
-			m.appClose <- item
+			appId := appInfo.GetId()
+			logger.Infof("current appId is %s", appId)
+			if m.shouldUseProxy(appId) {
+				pid := cmd.Process.Pid
+				logger.Infof("should use proxy, %v", pid)
+				err = m.appProxy.AddProc(0, int32(pid))
+				if err != nil {
+					logger.Warningf("add proc failed, err: %v", err)
+				}
+			}
 		}
-
+		err := cmd.Wait()
 		if err != nil {
 			logger.Warningf("%v: %v", cmd.Args, err)
 
@@ -776,6 +720,28 @@ func (m *StartManager) waitCmd(appInfo *desktopappinfo.DesktopAppInfo, cmd *exec
 	return nil
 }
 
+func removeProxy(sl []string) []string {
+	result := removeSl(sl, "auto_proxy")
+	result = removeSl(result, "http_proxy")
+	result = removeSl(result, "https_proxy")
+	result = removeSl(result, "ftp_proxy")
+	result = removeSl(result, "all_proxy")
+	result = removeSl(result, "SOCKS_SERVER")
+	result = removeSl(result, "no_proxy")
+	return result
+}
+
+func removeSl(sl []string, mem string) []string {
+	for index, elem := range sl {
+		if !strings.HasPrefix(elem, mem) {
+			continue
+		}
+		sl = append(sl[:index], sl[index+1:]...)
+		break
+	}
+	return sl
+}
+
 func isDEComponent(appInfo *desktopappinfo.DesktopAppInfo) bool {
 	isDEComponent, _ := appInfo.GetBool(desktopappinfo.MainSection, "X-Deepin-DEComponent")
 	return isDEComponent
@@ -1124,11 +1090,3 @@ func isAppInList(app string, apps []string) bool {
 	}
 	return false
 }
-
-func ignoreUseProxy(id string) bool {
-	// deepin-app-store and deepin-manual implement proxy themselves, should be ignored
-	if id == "deepin-app-store" || id == "deepin-manual" {
-		return true
-	}
-	return false
-}
